==> sat4j_2.0.4.orig.tar.gz: sat4j-2.0.4/org.sat4j.core/src/main/java/org/sat4j/minisat/constraints/cnf/LearntHTClause.java <==
/*******************************************************************************
 * SAT4J: a SATisfiability library for Java Copyright (C) 2004-2008 Daniel Le Berre
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU Lesser General Public License Version 2.1 or later (the
 * "LGPL"), in which case the provisions of the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of the LGPL, and not to allow others to use your version of
 * this file under the terms of the EPL, indicate your decision by deleting
 * the provisions above and replace them with the notice and other provisions
 * required by the LGPL. If you do not delete the provisions above, a recipient
 * may use your version of this file under the terms of the EPL or the LGPL.
 * 
 * Based on the original MiniSat specification from:
 * 
 * An extensible SAT solver. Niklas Een and Niklas Sorensson. Proceedings of the
 * Sixth International Conference on Theory and Applications of Satisfiability
 * Testing, LNCS 2919, pp 502-518, 2003.
 *
 * See www.minisat.se for the original solver in C++.
 * 
 *******************************************************************************/
package org.sat4j.minisat.constraints.cnf;

import static org.sat4j.core.LiteralsUtils.neg;
import org.sat4j.minisat.core.ILits;
import org.sat4j.specs.IVecInt;

public class LearntHTClause extends HTClause {

	public LearntHTClause(IVecInt ps, ILits voc) {
		super(ps, voc);
	}

	/**
     * 
     */
	private static final long serialVersionUID = 1L;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.sat4j.minisat.constraints.cnf.WLClause#register()
	 */
	public void register() {
		assert lits.length > 1;
		// looking for the literal to put in tail
		int maxi = 1;
		int maxlevel = voc.getLevel(lits[1]);
		for (int i = 2; i < lits.length; i++) {
			int level = voc.getLevel(lits[i]);
			if (level > maxlevel) {
				maxi = i;
				maxlevel = level;
			}
		}
		int l = lits[tail];
		lits[tail] = lits[maxi];
		lits[maxi] = l;

		// watch both head and tail literals.
		voc.attach(neg(lits[HEAD]), this);
		voc.attach(neg(lits[tail]), this);

	}

	public boolean learnt() {
		return true;
	}

	public void setLearnt() {
		// do nothing
	}

}

==> sat4j_2.0.5.orig.tar.gz: sat4j-2.0.5/org.sat4j.core/src/main/java/org/sat4j/minisat/constraints/cnf/LearntHTClause.java <==
/*******************************************************************************
 * SAT4J: a SATisfiability library for Java Copyright (C) 2004-2008 Daniel Le Berre
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU Lesser General Public License Version 2.1 or later (the
 * "LGPL"), in which case the provisions of the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of the LGPL, and not to allow others to use your version of
 * this file under the terms of the EPL, indicate your decision by deleting
 * the provisions above and replace them with the notice and other provisions
 * required by the LGPL. If you do not delete the provisions above, a recipient
 * may use your version of this file under the terms of the EPL or the LGPL.
 * 
 * Based on the original MiniSat specification from:
 * 
 * An extensible SAT solver. Niklas Een and Niklas Sorensson. Proceedings of the
 * Sixth International Conference on Theory and Applications of Satisfiability
 * Testing, LNCS 2919, pp 502-518, 2003.
 *
 * See www.minisat.se for the original solver in C++.
 * 
 *******************************************************************************/
package org.sat4j.minisat.constraints.cnf;

import static org.sat4j.core.LiteralsUtils.neg;
import org.sat4j.minisat.core.ILits;
import org.sat4j.specs.IVecInt;

public class LearntHTClause extends HTClause {

	public LearntHTClause(IVecInt ps, ILits voc) {
		super(ps, voc);
	}

	/**
     * 
     */
	private static final long serialVersionUID = 1L;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.sat4j.minisat.constraints.cnf.WLClause#register()
	 */
	public void register() {
		// looking for the literal to put in tail
		if (middleLits.length > 0) {
			int maxi = 0;
			int maxlevel = voc.getLevel(middleLits[0]);
			for (int i = 1; i < middleLits.length; i++) {
				int level = voc.getLevel(middleLits[i]);
				if (level > maxlevel) {
					maxi = i;
					maxlevel = level;
				}
			}
			if (maxlevel > voc.getLevel(tail)) {
				int l = tail;
				tail = middleLits[maxi];
				middleLits[maxi] = l;
			}
		}
		// attach both head and tail literals.
		voc.attach(neg(head), this);
		voc.attach(neg(tail), this);

	}

	public boolean learnt() {
		return true;
	}

	public void setLearnt() {
		// do nothing
	}

}
